#!/usr/bin/perl -w
######################################################################################
# This scripts backtranslates the amino acid alignment on to nucleotide sequences
# Author: Vikas Kumar
# To run this script: Aligned amino acid file and Corresponding nucleotide file is required
# Important : Sequence should be in "Single line" fasta file not in multi-line fasta file for both amino acid alignment and nucleotide sequence file
# Email: vikas.kumar@senckenberg.de
# Usage: perl backtranslate.pl AminoAcidAlignedFile Nucleotide_file OutputFile
######################################################################################

use strict;
use Data::Dumper;
my $usage = "\"Make sure first that the files are not multi-line fasta files, if it is so convert it to single line fasta file\"\nUsage: perl backtranslate.pl AminoAcidAlignedFile NucleotideFile OutputFile\n";
my $aa_aligned_file = shift or die($usage);      
my $nt_sequence_file = shift or die($usage);
my $output_file = shift or die($usage);


sub back_translate($$);

open(FILE,"$aa_aligned_file");
my %aa_aln;
my %nuc;
while (my $line=<FILE>){
        chomp $line;        
        if ($line=~/^>/){
                my $id=$line;
                $line=<FILE>;
                chomp $line;
		my $seq=$line;
                $aa_aln{$id}=$seq;
        }
}

open(FILE2,"$nt_sequence_file");
while (my $line2=<FILE2>){
        chomp $line2;
	if ($line2=~/^>/){
                my $id2=$line2;
                $line2=<FILE2>;
                chomp $line2;
		my $seq2=$line2;
                $nuc{$id2}=$seq2;
	}
}
 
open("OUT",">$output_file");
my %nucaln = back_translate(\%aa_aln, \%nuc);
    foreach (keys %nucaln) {
        	print OUT "$_\n";
		print OUT "$nucaln{$_}\n";
    }


sub back_translate($$)
{
    my %alignment = %{$_[0]};
    my %nt_seq = %{$_[1]};

    my %nt_aln;
    foreach my $spec (keys %alignment) {
        my $pos=0;

       for (my $i=0; $i<=length($alignment{$spec}); $i++) {
            my $char = substr($alignment{$spec},$i,1);

            if ($char eq "-") {
                $nt_aln{$spec} .= "---";
            }
            else {
                $nt_aln{$spec} .= substr($nt_seq{$spec}, $pos, 3);
                $pos+=3;
            }
            
        }
    }
    return (%nt_aln);
}
